;;; checkdoc-vs-flyspell.el --- turn off flyspell during checkdoc

;; Copyright 2010, 2011, 2013, 2014, 2015 Kevin Ryde
;;
;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: convenience, ispell, checkdoc
;; URL: http://user42.tuxfamily.org/checkdoc-vs-flyspell/index.html
;; EmacsWiki: Ispell

;; checkdoc-vs-flyspell.el is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License as published
;; by the Free Software Foundation; either version 3, or (at your option)
;; any later version.
;;
;; checkdoc-vs-flyspell.el is distributed in the hope that it will be
;; useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.


;;; Commentary:

;; This is a hack to make `checkdoc-ispell-lisp-words' work in M-x checkdoc
;; when flyspell-mode is enabled.
;;
;; As of Emacs 23.2, if flyspell-mode is active then checkdoc's spell
;; checking (per `checkdoc-spellcheck-documentation-flag') doesn't consider
;; `checkdoc-ispell-lisp-words' as correct.
;;
;; The problem is `checkdoc-ispell-init' doesn't send those extra words to
;; ispell if ispell is already running, which is so for flyspell-mode, and
;; quite possibly other uses of ispell.
;;
;; An M-x unload-feature checkdoc-vs-flyspell can undo the hack applied, but
;; only in Emacs 22 up.  Call `checkdoc-vs-flyspell-unload-function'
;; directly for earlier Emacs if necessary.
;;
;; The alternative is to stick in a "LocalWords:" of the things you use (see
;; `ispell-words-keyword').  But you'll get sick of repeating "alist" etc in
;; almost every Lisp file.

;;; Install:

;; To apply this put checkdoc-vs-flyspell.el in one of your `load-path'
;; directories and the following in your .emacs
;;
;;     (eval-after-load "checkdoc" '(require 'checkdoc-vs-flyspell))
;;
;; Just the `require' could be used instead if desired.

;;; Code:

;; Explicit dependency on advice.el since
;; `checkdoc-vs-flyspell-unload-function' needs `ad-find-advice' macro when
;; running not byte compiled, and that macro is not autoloaded.
(require 'advice)

(defadvice checkdoc-ispell-docstring-engine (around checkdoc-vs-flyspell
                                                    activate)
  "Temporarily turn off flyspell and ensure Ispell restarts."
  (let ((checkdoc-vs-flyspell--old (and (boundp 'flyspell-mode)
                                        flyspell-mode)))
    (unwind-protect
        (progn
          (if checkdoc-vs-flyspell--old (flyspell-mode 0))
          (ispell-kill-ispell t)
          ad-do-it)
      (if checkdoc-vs-flyspell--old (flyspell-mode 1)))))

(defun checkdoc-vs-flyspell-unload-function ()
  "Remove advice on `checkdoc'.
This is called by `unload-feature'."
  (when (ad-find-advice 'checkdoc-ispell-docstring-engine
                        'around 'checkdoc-vs-flyspell)
    (ad-remove-advice   'checkdoc-ispell-docstring-engine
                        'around 'checkdoc-vs-flyspell)
    (ad-activate        'checkdoc-ispell-docstring-engine))
  nil) ;; and do normal unload-feature actions too

;; LocalWords: flyspell LocalWords

(provide 'checkdoc-vs-flyspell)

;;; checkdoc-vs-flyspell.el ends here
