;;; html-smarter-href.el --- smarter html-href-anchor reading

;; no `?' for complete

;; don't go to absolute path under one-word complete



;; Copyright 2001, 2002, 2009, 2011, 2013, 2014, 2015, 2017 Kevin Ryde

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Version: 0
;; Keywords: convenience, hypermedia, html
;; URL: http://user42.tuxfamily.org/html-smarter-href/index.html
;; EmacsWiki: HtmlMode

;; html-smarter-href.el is free software; you can redistribute it
;; and/or modify it under the terms of the GNU General Public License as
;; published by the Free Software Foundation; either version 3, or (at your
;; option) any later version.
;;
;; html-smarter-href.el is distributed in the hope that it will be
;; useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You can get a copy of the GNU General Public License online at
;; <http://www.gnu.org/licenses/>.


;;; Commentary:
;; 

;;; Install:

;; Put html-smarter-href.el in one of your `load-path' directories and to
;; override the core `html-href-anchor' and `html-image' add the following
;; to your .emacs
;;
;;     (autoload 'html-smarter-href-override "html-smarter-href")
;;     (add-hook 'html-mode-hook 'html-smarter-href-override)
;;
;; To try it temporarily do M-x load-library html-smarter-href then the
;; "smarter" skeleton is M-x html-smarter-href-anchor.
;;
;; There's autoload cookies for the override function if you install via
;; `M-x package-install' or know how to use `update-file-autoloads'.

;;; History:

;; Version 1 - the first version.

;;; Code:

;;;###autoload
(defcustom html-smarter-href-roots nil
  "Local filesystem paths for web site roots.
When editing a file under one of these paths that path is taken
to be the root for absolute links.

For example if \"/var/www\" is a root then in a file under there
`html-smarter-href-anchor' will offer completions of \"/aaa/bbb\"
according to files \"/var/www/aaa/bbb...\"."

  :type  '(repeat directory)
  :link  '(url-link :tag "html-smarter-href.el home page"
                    "http://user42.tuxfamily.org/html-smarter-href/index.html")
  :group 'sgml)

;;-----------------------------------------------------------------------------
;; URL or link reading

(defvar html-smarter-href-history nil
  "URLs and links entered in `html-smarter-href-read'.")

(defun html-smarter-href-completion (str pred action)
  "Perform completion for `html-smarter-href-read'.
Filenames on the local filesystem are completed, with absolute
paths taken to begin at `html-smarter-href-roots'
if it's not nil.

PRED is currently ignored."

  ;; This is along the lines of the internal `completion--file-name-table',
  ;; but with trickery to go from `html-smarter-href-roots' if
  ;; set.
  (eval-and-compile
    (require 'cl))
  (let* ((basename (file-name-nondirectory str))
         (dirname  (or (file-name-directory str)
                       default-directory))
         (realdir  dirname))

    (when (and default-directory
               (string-match "\\`/" dirname))
      (some (lambda (root)
              (setq root (expand-file-name root))
              (if (string-match "\\'/" root) ;; lose trailing / from pattern
                  (setq root (substring root 0 -1)))
              ;; default-directory equal to or under root
              (and (string-match (concat "\\`" (regexp-quote root)
                                         "\\(\\'\\|/\\)")
                                 default-directory)
                   (setq realdir (concat root dirname))))
            html-smarter-href-roots))

    (cond ((eq action nil) ;; try-completion
           (let ((match (file-name-completion basename realdir)))
             (if (stringp match)
                 (concat dirname match)
               match)))
          ((eq action t)   ;; all-completions
           (file-name-all-completions basename realdir))
          (t               ;; test completion, no restrictions
           t))))

(defvar html-smarter-href-map
  (let ((m (make-sparse-keymap)))
    (define-key m " " 'self-insert-command)
    (define-key m "?" 'self-insert-command)
    m)
  "Keymap for minibuffer html link completion.
`html-smarter-href-read' let-binds
`minibuffer-local-completion-map' to this keymap, with the
original `minibuffer-local-completion-map' as the parent.  The
effect is to override the \"space\" and \"?\" bindings back to
plain `self-insert-command'.

A \"?\" is often found in CGI style urls.  A space is unusual,
but works, so entering it is allowed.  If you paste from the
console with gpm (not in `gpm-mouse-mode') then it's easy to
include some spaces.")

(defun html-smarter-href-fixup (str)
  (if (string-match "\\`\\s-+" str)  ;; strip leading whitespace
      (setq str (substring str (match-end 0))))
  (if (string-match "\\s-+\\'" str)  ;; strip trailing whitespace
      (setq str (substring str 0 (match-beginning 0))))

  ;; <...> strip surrounding angles
  ;; <URL:...>
  (if (string-match "\\`<\\(URL:\\)?\\(.*\\)>\\'" str)
      (setq str (match-string 2 str)))

  (let ((case-fold-search nil))
    (cond
     ;; http:/foo or ftp:/foo fixup
     ((string-match "\\`\\(http\\|ftp\\):/\\([^/]\\)" str)
      (setq str (concat (match-string 1) "://"
                        (substring str (match-beginning 1)))))

     ;; file:/foo or file://foo fixup
     ((string-match "\\`file:/{1,2}\\([^/]\\)" str)
      (setq str (concat "file:///" (substring str (match-beginning 1)))))

     ;; ftp.foo.org expand ftp://ftp.foo.org
     ((string-match "\\`ftp\\.[A-Za-z0-9_-]+\\." str)
      (setq str (concat "ftp://" str)))

     ;; foo.org/ expand http://foo.org/
     ((string-match "\\`[A-Za-z0-9_-]+\\(\\.[A-Za-z0-9_-]+\\)+/" str)
      (setq str (concat "http://" str)))))
  str)

(defun html-smarter-href-read ()
  "Read a URL link string from the user.
Filenames like \"../index.html\" or \"/some/dir/...\" on the
local filesystem can be completed with \\<minibuffer-local-completion-map>\\[minibuffer-complete] etc.
This is good for links within your own website.  See
`html-smarter-href-roots' for where your site paths
begin in the filesystem.

The string entered is massaged,

* Leading and trailing whitespace removed.

* \"foo.com\" without a protocol is assumed to be http and
  expands to \"http://foo.com\".

* \"ftp.foo.com\" is taken to be ftp and expands to
  \"ftp://ftp.foo.com\".

* A typo \"http:/foo\" is corrected to \"http://foo\".  Likewise
  \"ftp:/foo\" to \"ftp://foo\".

  \"file:/foo\" and \"file://foo\" are corrected to \"file:///foo\".
  It's possible \"file://foo\" is a hostname in the file schema,
  but in practice it's usually a typo.

There's no \"http:\" pre-loaded into the input.  Emacs 21 had
that and it was annoying most of the time.  It's gone in Emacs 22
up.

Incidentally, if you're thinking to have file:/// automatically
applied for local links in a personal start page, you almost
certainly don't need it.  If you access the page as
file:///home/fred/mystart.html then a link like
/usr/doc/something/manual.html uses file: too."

  (html-smarter-href-fixup
   (progn
     (set-keymap-parent html-smarter-href-map minibuffer-local-completion-map)
     (let ((minibuffer-local-completion-map html-smarter-href-map))
       (completing-read "URL: "                       ;; prompt
                        'html-smarter-href-completion ;; collection
                        nil   ;; predicate
                        nil   ;; require-match
                        ""    ;; initial
                        'html-smarter-href-history)))))

;;-----------------------------------------------------------------------------
;; skeletons

(define-skeleton html-smarter-href-anchor
  "HTML anchor tag with href attribute.
Similar to `html-href-anchor' but the URL or link is read with
`html-smarter-href-read' with its various completions and
manipulations."
  '(html-smarter-href-read)
  "<a href=\"" str "\">" _ "</a>")

(define-skeleton html-smarter-image
  "HTML image tag.
Similar to `html-image' but the URL or link is read with
`html-smarter-href-read' with its various completions and
manipulations."
  '(html-smarter-href-read)
  "<img src=\"" str "\" alt=\"" _ "\""
  (if sgml-xml-mode " />" ">"))

;;;###autoload
(defun html-smarter-href-override ()
  "Override `html-href-anchor' and `html-image'.
Those skeletons are changed to have the \"smarter\" URL and link
reading of `html-smarter-href-anchor' and `html-smarter-image'.

This override is an easy way to make all key bindings, menus,
etc, use the \"smarter\" versions."

  (require 'sgml-mode)
  ;; the same as `html-smarter-href-anchor' above
  (define-skeleton html-href-anchor
    "HTML anchor tag with href attribute.
(Definition replaced by `html-smarter-href-override'.)

A URL or link is read with `html-smarter-href-read' with
various completions and manipulations."
    '(html-smarter-href-read)
    "<a href=\"" str "\">" _ "</a>")

  (define-skeleton html-image
    "HTML image tag.
(Definition replaced by `html-smarter-href-override'.)

A URL or link is read with `html-smarter-href-read' with
various completions and manipulations."
    '(html-smarter-href-read)
    "<img src=\"" str "\" alt=\"" _ "\""
    (if sgml-xml-mode " />" ">")))

;;;###autoload
(custom-add-option 'html-mode-hook 'html-smarter-href-override)

;; LocalWords: www aaa bbb var filesystem Filenames minibuffer http html urls
;; LocalWords: dir hostname fred mystart usr gpm foo com

(provide 'html-smarter-href)

;;; html-smarter-href.el ends here
