;; Copyright 2006, 2007, 2009, 2010 Kevin Ryde

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defun my-get-diff-buffer ()
  "Wait for a `diff' to finish and return its buffer.
Emacs `diff' returns the buffer, but not `diff-buffer-with-file',
`vc-diff', nor XEmacs `diff'."

  (let (found)
    (mapcar (lambda (buffer)
              (if (string-match "Diff" (buffer-name buffer))
                  (setq found buffer)))
            (buffer-list))

    (unless found
      (error "my-get-diff-buffer: cannot find Diff buffer"))

    ;; diff runs in the background, wait for it to finish
    (let ((count 0))
      (while (and found
                  (get-buffer-process found)
                  (eq 'run (process-status found)))
        (setq count (1+ count))
        (if (> count 30)
            (error "oops, diff process not stopping"))
        (sit-for 1)))

    found))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(diff-save-buffer
              diff-save-buffer-keybinding))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((diff-mode-hook        diff-save-buffer-keybinding)
              (compilation-mode-hook diff-save-buffer-keybinding)))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'diff-save-buffer)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; diff-save-buffer-unquote

    (mapcar (lambda (elem)
              (let* ((obj (nth 0 elem))
                     (want (nth 1 elem))
                     (got  (diff-save-buffer-unquote obj)))
                (unless (equal got want)
                  (error "diff-save-buffer-unquote wrong, got %S want %S"
                         got want)))
              (setq count (1+ count)))

            '(("foo" "foo")
              ((quote "foo") "foo")))

    ;;-------------------------------------------------------------------------
    ;; `diff'

    (progn
      (diff "t-align-let.el" "t-diff-save-buffer.el")
      (switch-to-buffer (my-get-diff-buffer))

      (let ((got  (diff-save-buffer-filenames))
	    (want (cons (expand-file-name "t-align-let.el")
			(expand-file-name "t-diff-save-buffer.el"))))
	(unless (equal got want)
	  (error "wrong M-x diff filenames got %S want %S"
		 got want)))
      (setq count (1+ count))

      (let ((got  (diff-save-buffer-initial))
	    (want "t-diff-save-buffer.el.diff"))
	(unless (equal got want)
	  (error "wrong M-x diff initial got %S want %S"
		 got want)))
      (setq count (1+ count))

      (kill-buffer nil))

    ;;-------------------------------------------------------------------------
    ;; `diff-buffer-with-file'

    (if (not (fboundp 'diff-buffer-with-file))
	(message "skip no `diff-buffer-with-file'")

      (find-file "t-diff-save-buffer.el")
      (diff-buffer-with-file)
      (switch-to-buffer (my-get-diff-buffer))

      (let ((got  (diff-save-buffer-initial))
	    (want "t-diff-save-buffer.el.diff"))
	(unless (equal got want)
	  (error "wrong M-x diff-buffer-with-file initial got %S want %S"
		 got want)))
      (setq count (1+ count))

      (kill-buffer nil))

    ;;-------------------------------------------------------------------------
    ;; `vc-diff' would need rcs or something available

    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'diff-save-buffer))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; unload-feature -- read-file-name still works

    (require 'advice)
    (if (ad-is-advised 'read-file-name)
        (ad-deactivate 'read-file-name))
    (let ((ad-redefinition-action 'accept)
          (orig (symbol-function 'read-file-name))
          (want "1234"))
      (fset 'read-file-name (lambda (prompt &optional
                                            dir default-filename mustmatch
                                            initial predicate)
                              want))
      (if (ad-is-advised 'read-file-name)
          (ad-activate 'read-file-name))

      (require 'diff-save-buffer)
      (unload-feature 'diff-save-buffer t)

      (let ((got (read-file-name "test run read-file-name ... ")))
        (message "t-diff-save-buffer.el: read-file-name gave %S" got)
        (unless (equal got want)
          (error "read-file-name not quite right, maybe")))

      (if (ad-is-advised 'read-file-name)
          (ad-deactivate 'read-file-name))
      (fset 'read-file-name orig)
      (if (ad-is-advised 'read-file-name)
          (ad-activate 'read-file-name))

      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'diff-save-buffer))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))
