;; Copyright 2011, 2012, 2016 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


;; set this to `t' for some diagnostic messages
(defvar my-test-verbose nil)


(add-to-list 'load-path (expand-file-name ".."))

(unless (fboundp 'file-expand-wildcards)
  (load (expand-file-name "file-expand-wildcards.el")))

(defconst my-gzip-available-p
  (condition-case nil
      (equal 0 (call-process "gzip"
                             nil  ;; stdin
                             nil  ;; stdout
                             nil  ;; display
                             "--version"))
    (error nil))
  "Non-nil if the \"gzip\" program is available.")

(defun my-browse-url-error (url &rest args)
  "For use in `browse-url-browser-function'."
  (error "My browse-url throwing an error"))

(defun my-run-test ()
  (require 'miscbits-el-loaddefs)
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoload

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(wx-lookup-symbol))

    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'wx-lookup)
      (kill-buffer nil))


    ;;-------------------------------------------------------------------------
    ;; wx-lookup-completing-help-get

    (require 'wx-look) ;; before let-binding
    (let ((wx-lookup-cache '(("foo" . ("Foo.html" . "foo"))
                             ("bar" . ("Bar.html" . "bar")))))
      (mapcar (lambda (elem)
                (let* ((str  (nth 0 elem))
                       (want (nth 1 elem))
                       (got  (wx-lookup-completing-help-get str)))
                  (unless (equal got want)
                    (error "wx-lookup-completing-help-get %S got %S want %S"
                           str got want))
                  (setq count (1+ count))))

              '(("foo" "in Foo.html #foo")
                ("bar" "in Bar.html #bar")
                ("unknown" nil))))

    ;;-------------------------------------------------------------------------
    ;; wx-lookup--with-auto-compression

    (cond ((not (fboundp 'wx-lookup--with-auto-compression))
           (message "skip, no `wx-lookup--with-auto-compression' macro after byte compiled"))

          ((not my-gzip-available-p)
           (message "skip auto-compression test due to no gzip program"))

          (t
           (require 'wx-look)
           (let* ((plainfile "t-wx-look.tmp")
                  (gzfile    (concat plainfile ".gz")))
             (unwind-protect
                 (progn
                   (with-temp-buffer
                     (insert "hello world\n")
                     (write-file plainfile))
                   (call-process "gzip" nil nil nil plainfile)

                   (wx-lookup--with-auto-compression
                    (find-file gzfile)
                    (unless (equal "hello world\n" (buffer-string))
                      (error "wx-lookup--with-auto-compression didn't work"))
                    (kill-buffer nil) ;; gzfile
                    (setq count (1+ count))))

               (condition-case nil (delete-file plainfile) (error))
               (condition-case nil (delete-file gzfile)    (error))))))

    ;;-------------------------------------------------------------------------
    ;; wx-lookup-cache-init -- no &foo; entities

    (mapcar (lambda (elem)
              (let ((name (car elem))
                    (file (cadr elem))
                    (link (cddr elem)))
                (if (string-match "&.*;" name)
                    (error "Oops, unescaped entity in name: %S" elem))
                (if (string-match "&.*;" file)
                    (error "Oops, unescaped entity in file: %S" elem))
                (if (string-match "&.*;" link)
                    (error "Oops, unescaped entity in link: %S" elem))))
            (wx-lookup-cache-init))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; wx-lookup-canonicalize-symbol

    ;; pretend no indexes, so the tests don't vary with what you've got
    ;; installed
    (require 'wx-look) ;; before let-binding
    (let ((wx-lookup-cache
           '(("wxApp")
             )))

      (mapcar (lambda (elem)
                (let* ((str  (nth 0 elem))
                       (want (nth 1 elem))
                       (wx-lookup-cache (if (>= (length elem) 3)
                                            (nth 2 elem)
                                          wx-lookup-cache))
                       (got  (wx-lookup-canonicalize-symbol str)))

                  (if my-test-verbose
                      (message "str: %s" str))
                  (unless (string-equal want got)
                    (error "str %S got %S want %S, with cache %S"
                           str got want wx-lookup-cache))
                  (setq count (1+ count))))

              '(
                ("Wx::App"  "wxApp")
                ("Wx::App->new" "wxApp::wxApp")

                ;; Foo -> wxFoo
                ("App" "wxApp")
                )))


    ;;-------------------------------------------------------------------------
    ;; wx-lookup-string-suffix-ci-p

    (mapcar (lambda (elem)
              (let* ((suff (car elem))
                     (str  (car (cdr elem)))
                     (want (car (cdr (cdr elem))))
                     (got  (wx-lookup-string-suffix-ci-p suff str)))
                (unless (equal want got)
                  (error "suff %s str %s got %s want %s\n" suff str got want))
                (setq count (1+ count))))

            '(("new" "label_new" t)
              ("new" "NEW" t)
              ("longsuff" "str" nil)
              ("longsuff" "" nil)))


    ;;-------------------------------------------------------------------------
    ;; wx-lookup-cache

    (wx-lookup-reset)
    (wx-lookup-cache-init)
    (mapcar (lambda (elem)
              ;; (NAME . (FILE . LINK))
              (let* ((name (car elem))
                     (file (cadr elem))
                     (link  (cddr elem)))
                (when (string-match "\\`\\s-" name)
                  (error "cache entry \"name\" has leading whitespace: %S\n%S"
                         name elem))
                (when (and (string-match "\\`\\s-*\\S-+\\s-+\\'" name)
                           (not (string-match "::operator" name)))
                  (error "cache entry \"name\" has trailing whitespace: %S\n%S"
                         name elem))
                (unless (string-match "\\`[a-zA-Z_:<]" name)
                  (error "cache entry \"name\" non alpha start: %S\n%S"
                         name elem))
                (when (string-match "\\s-" link)
                  (error "cache entry \"link\" has whitespace: %S\n%S"
                         link elem))
                (when (string-match "\\s-" file)
                  (error "cache entry \"file\" has whitespace: %S\n%S"
                         file elem))))
            wx-lookup-cache)
    (message "wx-lookup-cache size %S" (length wx-lookup-cache))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------
    ;; wx-lookup-browse-url-other-window

    ;; check current buffer unchanged on error
    (require 'browse-url)
    (with-temp-buffer
      (let* ((want-buffer (current-buffer))
             (want-msg    "My browse-url throwing an error")
             (browse-url-browser-function (lambda (url &optional new-session)
                                            (error "%s" want-msg)))
             got-buffer
             got-msg)
        (condition-case err
            (wx-lookup-browse-url-other-window "file:///tmp")
          (error
           (setq got-msg (error-message-string err))))
        (setq got-buffer (current-buffer))

        (unless (equal got-msg want-msg)
          (error "browse-url error message got %S want %S"
                 got-msg want-msg))
        (unless (equal got-buffer want-buffer)
          (error "browse-url error buffer got %S want %S"
                 got-buffer want-buffer))
        (setq count (1+ count))))


    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (let ((prop '(wx-lookup-symbol bounds-of-thing-at-point)))
      (unless (apply 'get prop)
        (error "oops, didn't find thing-at-point property"))

      (unload-feature 'wx-look t)

      (if (featurep 'xemacs)
          (message "skip, xemacs21 unload-feature doesn't purge hook variables")
        (when (memq 'wx-lookup-display-initial-completion-list
                    minibuffer-setup-hook)
          (error "unload-feature leftover minibuffer-setup-hook %S"
                 minibuffer-setup-hook))
        (setq count (1+ count)))

      (add-to-list 'load-path (expand-file-name "."))
      (require 'my-unload)
      (if (not my-unload-feature-function-p)
          (message "skip, no unload-feature function")

        (let ((func (apply 'get prop)))
          (when func
            (error "unload-feature leftover %S property: %S" prop func)))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))
